# -*- coding: utf-8 -*-

thumb_suffix = '.tn'
thumb_size = (200, 150)
keep_orig = True

links = [('Example link', 'http://example.org'),
         ('Another link', 'http://example.org')]

copyright = "An example copyright message"
