/* Galleria Classic Theme 2012-08-07 | https://raw.github.com/aino/galleria/master/LICENSE | (c) Aino */

#galleria-loader{height:1px!important}

.galleria-container {
  position: relative;
  overflow: hidden;
}
.galleria-container img {
  -moz-user-select: none;
  -webkit-user-select: none;
  -o-user-select: none;
}
.galleria-stage {
  background: #222;
  background: rgba( 0, 0, 0, .15 );
  position: absolute;
  top: 0;
  bottom: 80px;
  left: 10px;
  right: 10px;
  overflow:hidden;
  -moz-border-radius: 4px;
  -webkit-border-radius: 4px;
  border-radius: 4px;
}
.galleria-thumbnails-container {
  height: 70px;
  bottom: 0;
  position: absolute;
  left: 10px;
  right: 10px;
  z-index: 2;
}
.galleria-carousel .galleria-thumbnails-list {
  margin-left: 30px;
  margin-right: 30px;
}
.galleria-thumbnails .galleria-image {
  height: 60px;
  width: 90px;
  background: transparent;
  margin: 0 10px 0 0;
  float: left;
  cursor: pointer;
}
.galleria-counter {
  position: absolute;
  bottom: 10px;
  left: 10px;
  text-align: right;
  z-index: 2;
}
.galleria-loader {
  width: 20px;
  height: 20px;
  position: absolute;
  top: 10px;
  right: 10px;
  z-index: 2;
  display: none;
  background: url('../img/classic-loader.gif') no-repeat 2px 2px;
}
.galleria-info {
  width: 50%;
  top: 15px;
  left: 25px;
  z-index: 2;
  position: absolute;
}
.galleria-info-text {
  background: #000;
  background: rgba(0, 0, 0, .4);
  padding: 15px;
  display: none;
  -moz-border-radius: 4px;
  -webkit-border-radius: 4px;
  border-radius: 4px;
}
.galleria-info-title {
  font-size: 1.4em;
  line-height: 1.4em;
  font-weight: bold;
  margin: 0;
  color: #fff;
}
.galleria-info-description {
  font-size: 1.2em;
  line-height: 1.2em;
  margin: 0;
  color: #bbb;
}
.galleria-info-title+.galleria-info-description { margin-top: 7px; }
.galleria-info-close {
  width: 9px;
  height: 9px;
  position: absolute;
  top: 10px;
  right: 10px;
  background-position: -753px -11px;
  opacity: .5;
  filter: alpha(opacity=50);
  cursor: pointer;
  display: none;
}
.notouch .galleria-info-close:hover{
  opacity:1;
  filter: alpha(opacity=100);
}
.touch .galleria-info-close:active{
  opacity:1;
  filter: alpha(opacity=100);
}
.galleria-info-link {
  background-position: -669px -5px;
  opacity: .7;
  filter: alpha(opacity=70);
  position: absolute;
  width: 20px;
  height: 20px;
  cursor: pointer;
}
.notouch .galleria-info-link:hover {
  opacity: 1;
  filter: alpha(opacity=100);
}
.touch .galleria-info-link:active {
  opacity: 1;
  filter: alpha(opacity=100);
}
.galleria-image-nav {
  position: absolute;
  top: 50%;
  margin-top: -62px;
  width: 100%;
  height: 62px;
  left: 0;
}
.galleria-image-nav-left,
.galleria-image-nav-right {
  opacity: .3;
  filter: alpha(opacity=30);
  cursor: pointer;
  width: 62px;
  height: 124px;
  position: absolute;
  left: 15px;
  z-index: 2;
  background-position: 0 46px;
}
.galleria-image-nav-right {
  left: auto;
  right: 15px;
  background-position: -254px 46px;
  z-index: 2;
}
.notouch .galleria-image-nav-left:hover,
.notouch .galleria-image-nav-right:hover {
  opacity: 1;
  filter: alpha(opacity=100);
}
.touch .galleria-image-nav-left:active,
.touch .galleria-image-nav-right:active {
  opacity: 1;
  filter: alpha(opacity=100);
}
.galleria-thumb-nav-left,
.galleria-thumb-nav-right {
  cursor: pointer;
  display: none;
  background-position: -495px 5px;
  position: absolute;
  left: 0;
  top: 0;
  height: 40px;
  width: 23px;
  z-index: 3;
  opacity: .8;
  filter: alpha(opacity=80);
}
.galleria-thumb-nav-right {
  background-position: -578px 5px;
  border-right: none;
  right: 0;
  left: auto;
}
.galleria-thumbnails-container .disabled {
  opacity: .2;
  filter: alpha(opacity=20);
  cursor: default;
}
.notouch .galleria-thumb-nav-left:hover,
.notouch .galleria-thumb-nav-right:hover {
  opacity: 1;
  filter: alpha(opacity=100);
  background-color: #111;
}
.touch .galleria-thumb-nav-left:active,
.touch .galleria-thumb-nav-right:active {
  opacity: 1;
  filter: alpha(opacity=100);
  background-color: #111;
}
.notouch .galleria-thumbnails-container .disabled:hover {
  opacity: .2;
  filter: alpha(opacity=20);
  background-color: transparent;
}

.galleria-carousel .galleria-thumb-nav-left,
.galleria-carousel .galleria-thumb-nav-right {
  display: block;
}
.galleria-thumb-nav-left,
.galleria-thumb-nav-right,
.galleria-info-link,
.galleria-info-close,
.galleria-image-nav-left,
.galleria-image-nav-right {
  background-image: url('../img/classic-map.png');
  background-repeat: no-repeat;
}
