<!doctype html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">

    <title>{{ title }}</title>
    <meta name="description" content="">
    <meta name="author" content="{{ author }}">
    <meta name="viewport" content="width=device-width">

    <link rel="stylesheet" href="http://fonts.googleapis.com/css?family=Molengo">
    <link rel="stylesheet" href="{{ theme.url }}/css/style.min.css">
    <!--[if lt IE 9]>
	<script src="http://html5shiv.googlecode.com/svn/trunk/html5.js"></script>
    <![endif]-->
  </head>
  <body>
    <div class="container">
      <header>
        <h1><a href="{{ index_url }}">{{ index_title }}</a></h1>

        {% if settings.links %}
        <nav id="menu">
          <ul>
            {% for title, link in settings.links %}
            <li><a href="{{ link }}">{{ title }}</a></li>
            {% endfor %}
          </ul>
        </nav>
        {% endif %}

        {% if breadcumb %}
        <h2>
          {%- for url, title in breadcumb -%}
          <a href="{{ url }}">{{ title }}</a>
          {%- if not loop.last %} » {% endif -%}
          {% endfor -%}
        </h2>
        <hr>
        {% endif %}
      </header>

      <div id="main" role="main">
        {% if albums %}
        <div id="albums">
          <!-- <h1>Albums</h1> -->
          <ul>
            {% for album in albums %}
            <li><a href="{{ album.url }}">
              <img src="{{ album.thumb }}" class="album_thumb" alt="{{ album.name }}" title="{{ album.name }}" /></a>
              <span class="album_title">{{ album.title }}</span>
            </li>
            {% endfor %}
          </ul>
        </div>
        {% endif %}

        {% if images %}
        <div id="gallery">
          {% for image in images %}
          <a href="{{ image.file }}">
            <img src="{{ image.thumb }}" alt="{{ image.file }}"
                 data-title="{{ image.file }}" /></a>
          {% endfor %}
        </div>
        {% endif %}

        {% if description %}
        <div id="description">
          {{ description }}
        </div>
        {% endif %}
      </div>

      <footer>
        <p>{% if author %}&copy; {{ author }} - {% endif %}
          Generated by <a href="{{ sigal_link }}">sigal</a></p>
      </footer>
    </div>

    {% if images %}
    <script src="//ajax.googleapis.com/ajax/libs/jquery/1.8.2/jquery.min.js"></script>
    <script>!window.jQuery && document.write(unescape('%3Cscript src="{{ theme.url }}/js/jquery-1.8.2.min.js"%3E%3C/script%3E'))</script>
    <script src="{{ theme.url }}/js/galleria-1.2.9.min.js"></script>
    <script src="{{ theme.url }}/js/galleria.classic.min.js"></script>
    <script>
    Galleria.configure({
        imageCrop: false,
        transition: "fade"
    });
    Galleria.run("#gallery");
    </script>
    {% endif %}
    {% include 'analytics.html' %}
  </body>
</html>
